DEVICE=1
_globalVar={} # 全局变量, 为避免过多及混乱, 全部储存在一个字典里
_globalVar['device']=DEVICE # 本地电脑为1, 服务器为0, 该值不可在函数中修改
_globalVar['cookieWrongTimes']=0
# GetCourseHeader
_globalVar['gch']={
    "Host": "coursesel.umji.sjtu.edu.cn",
    "Connection": "keep-alive",
    "sec-ch-ua": "\" Not A;Brand\";v=\"99\", \"Chromium\";v=\"96\", \"Google Chrome\";v=\"96\"",
    "Accept": "application/json, text/javascript, */*; q=0.01",
    "X-Requested-With": "XMLHttpRequest",
    "sec-ch-ua-mobile": "?0",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
    "sec-ch-ua-platform": "\"macOS\"",
    "Sec-Fetch-Site": "same-origin",
    "Sec-Fetch-Mode": "cors",
    "Sec-Fetch-Dest": "empty",
    "Referer": "https://coursesel.umji.sjtu.edu.cn/welcome.action",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9",
    #"Cookie": "_ga=GA1.3.1749969549.1634711853; JSESSIONID=8745FC6AB05D0CB11E5332024E3E04BE" # 这里的 Cookie 已失效
    "Cookie": "_ga=GA1.3.1749969549.1634711853; JSESSIONID="
}
# SelectCourseHeader
_globalVar['sch']={
    "Host": "coursesel.umji.sjtu.edu.cn",
    "Connection": "keep-alive",
    "Content-Length": "252", # 此项需根据实际长度修改
    "sec-ch-ua": "\" Not A;Brand\";v=\"99\", \"Chromium\";v=\"96\", \"Google Chrome\";v=\"96\"",
    "X-CSRF-TOKEN": "01bfe76b-74d8-4136-9ec5-bac2e7d991ac",
    "sec-ch-ua-mobile": "?0",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
    "Content-Type": "application/x-www-form-urlencoded",
    "Accept": "application/json, text/javascript, */*; q=0.01",
    "X-Requested-With": "XMLHttpRequest",
    "sec-ch-ua-platform": "\"macOS\"",
    "Origin": "https://coursesel.umji.sjtu.edu.cn",
    "Sec-Fetch-Site": "same-origin",
    "Sec-Fetch-Mode": "cors",
    "Sec-Fetch-Dest": "empty",
    "Referer": "https://coursesel.umji.sjtu.edu.cn/welcome.action",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9",
    #"Cookie": "_ga=GA1.3.1749969549.1634711853; JSESSIONID=8745FC6AB05D0CB11E5332024E3E04BE" # 这里的 Cookie 已失效
    "Cookie": "_ga=GA1.3.1749969549.1634711853; JSESSIONID="
}
# ElectTurnIdUrl
_globalVar['etiu']='https://coursesel.umji.sjtu.edu.cn/tpm/findStudentElectTurns_ElectTurn.action?_t='
# SelectCourseUrl
_globalVar['scu']="https://coursesel.umji.sjtu.edu.cn/tpm/doElect_ElectTurn.action?_t="
# GetCourseUrl
_globalVar['gcu1']="https://coursesel.umji.sjtu.edu.cn/tpm/findLessonTasks_ElectTurn.action?jsonString="
_globalVar['gcu2']='{"isToTheTime":true,"electTurnId":"'
_globalVar['gcu3']='","loadCourseGroup":true,"loadElectTurn":true,"loadCourseType":true,"loadCourseTypeCredit":true,"loadElectTurnResult":true,"loadStudentLessonTask":true,"loadPrerequisiteCourse":true,"lessonCalendarWeek":false,"loadLessonCalendarConflict":false,"loadTermCredit":true,"loadLessonTask":true,"loadDropApprove":true,"loadElectApprove":true}'

# ElectTurnId
_globalVar['eti']=''
_globalVar['mode']=''
_globalVar['electName']=''
_globalVar['startTime']=0
_globalVar['endTime']=0
_globalVar['newMsg']=False
_globalVar['allInfo']={}
_globalVar['failTimes']=0
_globalVar['IntFailTimes']=0
_globalVar['courses']=tuple()

# globalVar 中还有另外一种数据类型:
# key 为 courseId, value 为 [electTurnLessonTaskId,canSelect,description]

def getGlobalValue(key):
    global _globalVar
    return _globalVar[key]


def setGlobalValue(key,v):
    global _globalVar
    _globalVar[key]=v


def getAllGlobalVariables():
    global _globalVar
    return _globalVar

def getGlobalVariableNum():
    global _globalVar
    return len(_globalVar)

def clearGlobalVariable():
    global _globalVar
    _globalVar={}

def deleteGlobalVariable(key):
    global _globalVar
    try:
        _globalVar.pop(key)
    except:
        return -1
    return 0


def getAllKeys():
    global _globalVar
    l=[]
    for k in _globalVar:
        l.append(k)
    return l

def isGlobalVariableExist(key):
    global _globalVar
    return key in _globalVar


def addGlobalVariable(key,num=1):
    global _globalVar
    try:
        _globalVar[key]=_globalVar[key]+num
    except:
        return -1
    return 0




from .main import help,check,printCourseList,waitEmptySpace,fastSelect,luckyDraw

__all__=[
    'help',
    'check',
    'printCourseList',
    'waitEmptySpace',
    'fastSelect',
    'luckyDraw'
]


__version__='0.9.0'
__license__='GPL v2'
__author__='Tiancheng Jiao'
__github__='https://github.com/jtc1246/courseSelector'
__author_email__='jtc1246@outlook.com'
__description__='Automatic course selection program for SJTU-JI'





from .globalVariable import *
import time
from copy import deepcopy
from .myHttp import *
import os


# 格式:
# {"electTurnId":"158C59EB-C733-4C3A-8E78-C314DCEC70BF","autoElect":true,"lessonTasks":["A57528CC-AECA-4266-874E-039386315A2A"]}
def getJsonString(electTurnId,electTurnLessonTaskId):
    return '{"electTurnId":"'+electTurnId+'","autoElect":true,"lessonTasks":["'+electTurnLessonTaskId+'"]}'


def getJsonStringLucky(electTurnId,electTurnLessonTaskId,lessonTaskId):
    return "{\"electTurnId\":\""+electTurnId+"\",\"lessonTasks\":[{\"electTurnLessonTaskId\":\""+electTurnLessonTaskId+"\",\"lessonTaskId\":\""+lessonTaskId+"\"}]}"



def getElectTurnId():
    url=getGlobalValue('etiu')+str(getTime())
    header=getGlobalValue('gch')
    r=http(url,Header=header,Timeout=1500)
    if(r['status']==-1):
        print('无网络连接, 程序已结束运行')
        os._exit(-1)
    if(r['status']==-2):
        print('连接服务器超时, 程序已结束运行')
        os._exit(-1)
    if(r['code']==302):
        print("Cookie 无效, 程序已结束运行")
        os._exit(-1)
    eti=r['text'][0]["electTurnId"]
    mode=r['text'][0]["electModeName"]
    name=r['text'][0]["electTurnName"]
    setGlobalValue('eti',eti)
    setGlobalValue('mode',mode)
    setGlobalValue('electName',name)
    startTime=toUnix(r['text'][0]["beginTime"])
    endTime=toUnix(r['text'][0]["endTime"])
    setGlobalValue('startTime',startTime)
    setGlobalValue('endTime',endTime)




def setCookie(jsessionid):
    gch=getGlobalValue('gch')
    sch=getGlobalValue('sch')
    gch=deepcopy(gch)
    sch=deepcopy(sch)
    gch['Cookie']=gch['Cookie']+jsessionid
    sch['Cookie']=sch['Cookie']+jsessionid
    setGlobalValue('gch',gch)
    setGlobalValue('sch',sch)


def getAllInformationFirst(): # 第一次获取信息时调用这个函数, 出现异常直接结束运行
    gcu1=getGlobalValue('gcu1')
    gcu2=getGlobalValue('gcu2')
    gcu3=getGlobalValue('gcu3')
    eti=getGlobalValue('eti')
    url=quote(gcu2+eti+gcu3)
    url=gcu1+url
    header=getGlobalValue('gch')
    r=http(url,Header=header,Timeout=3000)
    if(r['status']==-1):
        print('无网络连接, 程序已结束运行')
        os._exit(-1)
    if(r['status']==-2):
        print('连接服务器超时, 程序已结束运行')
        os._exit(-1)
    if(r['code']==302):
        print("Cookie 失效, 请重新设置 Cookie, 程序已结束运行")
        os._exit(-1)
    setGlobalValue('allInfo',r['text'])
    return r['text']


def getAllInformation(): # 后续循环时调用这个函数, 出现异常直接忽略
    # 这里先不设置循环, 后面将这个函数包在另一个函数里, 在那里设置循环
    gcu1=getGlobalValue('gcu1')
    gcu2=getGlobalValue('gcu2')
    gcu3=getGlobalValue('gcu3')
    eti=getGlobalValue('eti')
    url=quote(gcu2+eti+gcu3)
    url=gcu1+url
    header=getGlobalValue('gch')
    r=http(url,Header=header,Timeout=3000)
    if(r['code']==302):
        print(str(getTime())+": Cookie 失效, 请重新设置 Cookie, 程序已结束运行")
        os._exit(-1)
    if(r['status']!=0 or r['status']==3):
        return -1
    try:
        re=r['text']['data']
    except:
        return -1
    return r['text']


def getSelectedCoursesId(AllInformation):
    all=AllInformation['data']["electTurnResult"]
    l=[]
    for one in all:
        l.append(one[['courseId']])
    return l

def selectOneCourse(courseId):
    while True:
        status=getGlobalValue(courseId)
        if(not status[1]):
            time.sleep(0.01)
            continue
        # 以下是要选的情况
        electTurnLessonTaskId=status[0]
        eti=getGlobalValue('eti')
        jss=getJsonString(eti,electTurnLessonTaskId)
        header=getGlobalValue('sch')
        header=deepcopy(header)
        body="jsonString="+quote(jss,encoding='utf-8')
        header['Content-Length']=str(len(body))
        url=getGlobalValue('scu')+str(getTime())
        r=http(url,Method='POST',Header=header,Timeout=500,BODY=body)



def selectOneCourseLucky(courseId):
    status=getGlobalValue(courseId)
    # 以下是要选的情况
    electTurnLessonTaskId=status[0]
    lessonTaskId=status[3]
    eti=getGlobalValue('eti')
    jss=getJsonStringLucky(eti,electTurnLessonTaskId,lessonTaskId)
    header=getGlobalValue('sch')
    header=deepcopy(header)
    body="jsonString="+quote(jss,encoding='utf-8')
    header['Content-Length']=str(len(body))
    url=getGlobalValue('scu')+str(getTime())
    r=http(url,Method='POST',Header=header,Timeout=1000,BODY=body)


def testInternetConnection():
    setGlobalValue('IntFailTimes',0)
    while True:
        re=testInternet()
        if(re==-1):
            setGlobalValue('IntFailTimes',getGlobalValue('IntFailTimes')+1)
        if(re==0):
            setGlobalValue('IntFailTimes',0)
        if(getGlobalValue('IntFailTimes')%10==3):
            print(str(getTime())+': 无法连接到互联网, 请检查网络连接')
        time.sleep(0.5)


def keepCookie():
    header=getGlobalValue('gch')
    url='https://coursesel.umji.sjtu.edu.cn/welcome.action'
    while True:
        r=http(url,Header=header,ToJSON=False)
        if(r['code']==302):
            print(str(getTime())+": Cookie 失效, 请重新设置 Cookie, 程序已结束运行")
            os._exit(-1)
        time.sleep(30)


def isSelected(courseId):
    allInfo=getGlobalValue('allInfo')
    selectedCoursesId=getSelectedCoursesId(allInfo)
    return courseId in selectedCoursesId


def hasSpace(courseId):
    allInfo=deepcopy(getGlobalValue('allInfo'))
    allInfo=allInfo['data']["lessonTasks"]
    for one in allInfo:
        if(one["courseId"]==courseId):
            maxNum=int(one["maxNum"])
            stuNum=int(one["studentNum"])
            return stuNum<maxNum
    return False

def getInfoMain():
    while True:
        if(getGlobalValue('failTimes')%20==19):
            print(str(getTime())+': 选课网站连接错误次数过多, 请检查原因, 如果当前是选课高峰期, 可忽略此问题')
        t=getAllInformation()
        if(t==-1):
            addGlobalVariable('failTimes')
            continue
        setGlobalValue('allInfo',t)
        setGlobalValue('newMsg',True)
        setGlobalValue('failTimes',0)




def mainControl():
    while True:
        if(not getGlobalValue('newMsg')):
            time.sleep(0.01)
            continue
        courses=getGlobalValue('courses')
        setGlobalValue('newMsg',False)
        for cid in courses:
            if(getGlobalValue(cid)[1]):
                # canSelect=True, 人数满、已选成功 满足任意一个即停止
                if(isSelected(cid)):
                    info=getGlobalValue(cid)
                    info=deepcopy(info)
                    print(str(getTime())+': '+info[2]+' 选课成功')
                    info[1]=False
                    setGlobalValue(cid,info)
                    continue
                if(not hasSpace(cid)):
                    info=getGlobalValue(cid)
                    info=deepcopy(info)
                    print(str(getTime())+': '+info[2]+' 选课失败')
                    info[1]=False
                    setGlobalValue(cid,info)
                    continue
            if(not getGlobalValue(cid)[1]):
                # canSelect=False, 未选择、有空位 两者同时满足才开始
                if( (not isSelected(cid)) and hasSpace(cid)):
                    info=getGlobalValue(cid)
                    info=deepcopy(info)
                    print(str(getTime())+': '+info[2]+' 发现空位')
                    info[1]=True
                    setGlobalValue(cid,info)
                    continue




# [electTurnLessonTaskId,canSelect,description]


from .globalVariable import *
import urllib3 as _myHttp_urllib3
import urllib3.exceptions as _myHttp_urllib3_exceptions
import time,json
from urllib.parse import quote
def testInternet():
    http1=_myHttp_urllib3.PoolManager()
    try:
        r=http1.request("GET","https://www.baidu.com",timeout=1.5,retries=False)
    except:
        return -1
    return 0

def http(url:str,Method:str="GET",Header:dict={},Timeout:int=0,ToJSON:bool=True,BODY:str="",Decode:bool=True):
    # status:  -1: 无网络  -2: 超时  -3: 域名不存在  -4: 其它问题, 主要是代理服务器设置错误(服务器上无此问题)
    # status:  1: 不是 UTF-8 编码, text 返回空字符串  2: 不是json格式(在toJSON=True的前提下)
    # status:  3: 对方网站不支持https, 但是却使用了https连接, 这种情况会自动切换为http连接, 若此次status不为0, 返回该status, 若为0, 返回3
    backup=[url,Method,Header,Timeout,ToJSON,BODY,Decode]
    http1=_myHttp_urllib3.PoolManager()
    global globalVar
    if(Timeout==0):
        Timeout=500+500*getGlobalValue('device')
    r=0
    if(ToJSON):
        text={}
    else:
        text=""
    try:
        r=http1.request(Method,url,headers=Header,timeout=Timeout/1000,body=BODY,retries=False)
    except _myHttp_urllib3_exceptions.NewConnectionError as err: # 无网络/域名不存在
        if(testInternet()==-1): # 无网络
            return {'status':-1,'code':0,'text':text,'header':{},'extra':''}
        return {'status':-3,'code':0,'text':text,'header':{},'extra':''}
    except _myHttp_urllib3_exceptions.ConnectTimeoutError as err: # 无网络/超时
        if(testInternet()==-1): # 无网络
            return {'status':-1,'code':0,'text':text,'header':{},'extra':''}
        # 超时
        try:
            r=http1.request(Method,url,headers=Header,timeout=3*Timeout/1000,body=BODY,retries=False)
        except:
            return {'status':-2,'code':0,'text':text,'header':{},'extra':''}
    except _myHttp_urllib3_exceptions.ReadTimeoutError as err: # 无网络/超时
        if(testInternet()==-1): # 无网络
            return {'status':-1,'code':0,'text':text,'header':{},'extra':''}
        # 超时
        try:
            r=http1.request(Method,url,headers=Header,timeout=3*Timeout/1000,body=BODY,retries=False)
        except:
            return {'status':-2,'code':0,'text':text,'header':{},'extra':''}
    except _myHttp_urllib3_exceptions.SSLError: # 对方网站不支持https, 但是却使用了https连接
        n=backup[0].find('https://')
        newUrl='http://'+backup[0][n+8:]
        tryAgain=http(newUrl,Method=backup[1],Header=backup[2],Timeout=backup[3],ToJSON=backup[4],BODY=backup[5],Decode=backup[6])
        if(tryAgain['status']==0):
            tryAgain['status']=3
        return tryAgain
    except Exception as err: # 其它错误，主要为代理服务器设置错误，服务器上一般无此问题
        return {'status':-4,'code':0,'text':text,'header':{},'extra':''}
    # 以下是正常情况
    resp={}
    resp['status']=0
    resp['code']=r.status
    respHeader=dict(r.headers)
    resp['header']=respHeader
    resp['extra']=''
    text=r.data
    if(not Decode):
        resp['text']=text
        return resp
    deco=''
    try:
        deco=text.decode('utf-8')
    except:
        resp['text']=''
        resp['status']=1
        resp['extra']=text
        return resp
    if(not ToJSON):
        resp['text']=deco
        return resp
    js={}
    try:
        js=toJson(deco)
    except:
        resp['text']={}
        resp['status']=2
        resp['extra']=deco
        return resp
    resp['text']=js
    return resp

def getTime():
    global globalVar
    t = time.time()
    t = int(1000 * t)
    return t


def toJson(Text):
    text=Text
    i=0
    while text[0]==' ' or text[0]=='\n':
        text=text[1:]
    if text[0]=='[':
        text='{"1":'+text+'}'
        i=1
    text=json.loads(text,strict=False)
    if i==1:
        text=text['1']
    return text


# 格式示例: "2022-01-17 10:00"
def toUnix(timeStr):
    timeStr=timeStr+':00'
    b=time.strptime(timeStr,'%Y-%m-%d %H:%M:%S')
    c=time.mktime(b)
    return int(c*1000)





from setuptools import setup

setup(
    name='courseSelector',
    version='0.9.1',
    author='Tiancheng Jiao',
    author_email='jtc1246@outlook.com',
    url='https://github.com/jtc1246/courseSelector',
    description='Automatic course selection program for SJTU-JI',
    packages=['courseSelector'],
    install_requires=['urllib3'],
    python_requires='>=3',
    platforms=["all"],
    license='GPL-2.0 License',
    entry_points={
        'console_scripts': [
            'help=courseSelector:help',
            'check=courseSelector:check',
            'printCourseList=courseSelector:printCourseList',
            'waitEmptySpace=courseSelector:waitEmptySpace',
            'fastSelect=courseSelector:fastSelect',
            'luckyDraw=courseSelector:luckyDraw'
        ]
    }
)


# __all__=[
#     'help',
#     'check',
#     'printCourseList',
#     'waitEmptySpace',
#     'fastSelect',
#     'luckyDraw'
# ]


from .myHttp import *
from .functions import *
from .globalVariable import *
import _thread as thread


def help():
    # 具体内容还没写
    print('欢迎使用本软件')






def check(jsessionid:str):
    setCookie(jsessionid)
    getElectTurnId()
    print('当前选课名称为: '+getGlobalValue('electName'))
    print('当前规则为: '+getGlobalValue('mode'))
    print('开始时间为: '+str(getGlobalValue('startTime')))
    print('结束时间为: '+str(getGlobalValue('endTime')))
    print('当前时间为: '+str(getTime()))


def printCourseList(jsessionid:str):
    check(jsessionid)
    getElectTurnId()
    all=getAllInformationFirst()
    all=all['data']["lessonTasks"]
    f=open('./courses.txt','w')
    f.write('课程名称      |      上课教师        |        课程描述\n')
    for one in all:
        print(one["courseId"])
        f.write(one["courseName"])
        f.write(' | ')
        try:
            f.write(one["lessonTaskTeam"])
        except:
            f.write('None')
        f.write(' | ')
        f.write(one["lessonClassName"])
        f.write('\n')
    f.close()
    print('获取课程列表成功, 请打开当前文件夹中的 course.txt 查看')
    
def waitEmptySpace(jsessionid:str,courses:list,threadNum:int=5):
    # 初始化部分
    if(threadNum<=0):
        print('线程数必须大于 0')
        os._exit(-1)
    check(jsessionid)
    if(getGlobalValue('mode').find('arly')==-1):
        print('本轮选课不是 Early Bird, 不适用本软件')
        os._exit(-1)
    all=deepcopy(getAllInformationFirst())
    all=all['data']["lessonTasks"]
    cids=[]
    isExit=False
    isFound=False
    for c in courses:
        isFound=False
        for one in all:
            if(one["lessonClassName"].find(c)>=0):
                cids.append(one['courseId'])
                setGlobalValue(one['courseId'],[one['electTurnLessonTaskId'],False,c])
                isFound=True
                break
        if(not isFound):
            isExit=True
            print('未找到该课程: '+c)
    if(isExit):
        print('存在未找到的课程, 程序已结束运行')
    cids=tuple(cids)
    setGlobalValue('courses',cids)
    # 初始化结束
    thread.start_new_thread(testInternetConnection,())
    thread.start_new_thread(keepCookie,())
    currentTime=getTime()
    if(currentTime<getGlobalValue('startTime')-10000):
        time.sleep(0.001*(-currentTime+getGlobalValue('startTime')-10000))
    for i in range(0,threadNum):
        thread.start_new_thread(getAllInformation,())
        time.sleep(1.4/threadNum)
    thread.start_new_thread(mainControl,())
    for i in cids:
        thread.start_new_thread(selectOneCourse,(i,))
    while True:
        time.sleep(10)
        isAllSelected=True
        for i in cids:
            if(not isSelected(i)):
                isAllSelected=False
        if(isAllSelected):
            print('要选课课程已经全部选择成功, 程序结束运行')
            os._exit(0)
        if(getTime()>getGlobalValue('endTime')+10000):
            print('选课已经结束, 程序结束运行')
            os._exit(0)




def fastSelect(jsessionid:str,courses:list,threadNum:int=10):
    # 初始化部分
    if(threadNum<=0):
        print('线程数必须大于 0')
        os._exit(-1)
    check(jsessionid)
    if(getGlobalValue('mode').find('arly')==-1):
        print('本轮选课不是 Early Bird, 不适用本软件')
        os._exit(-1)
    all=deepcopy(getAllInformationFirst())
    all=all['data']["lessonTasks"]
    cids=[]
    isExit=False
    isFound=False
    for c in courses:
        isFound=False
        for one in all:
            if(one["lessonClassName"].find(c)>=0):
                cids.append(one['courseId'])
                setGlobalValue(one['courseId'],[one['electTurnLessonTaskId'],True,c])
                isFound=True
                break
        if(not isFound):
            isExit=True
            print('未找到该课程: '+c)
    if(isExit):
        print('存在未找到的课程, 程序已结束运行')
    cids=tuple(cids)
    setGlobalValue('courses',cids)
    # 初始化结束
    thread.start_new_thread(testInternetConnection,())
    thread.start_new_thread(keepCookie,())
    currentTime=getTime()
    if(currentTime<getGlobalValue('startTime')-10000):
        time.sleep(0.001*(-currentTime+getGlobalValue('startTime')-10000))
    for i in range(0,3):
        thread.start_new_thread(getAllInformation,())
    thread.start_new_thread(mainControl,())
    for i in cids:
        for j in range(0,threadNum):
            thread.start_new_thread(selectOneCourse,(i,))
    while True:
        time.sleep(10)
        isAllSelected=True
        for i in cids:
            if(not isSelected(i)):
                isAllSelected=False
        if(isAllSelected):
            print('要选课课程已经全部选择成功, 程序结束运行')
            os._exit(0)
        if(getTime()>getGlobalValue('endTime')+10000):
            print('选课已经结束, 程序结束运行')
            os._exit(0)


# 在 Lucky Draw 模式下选时间冲突的课程
def luckyDraw(jsessionid:str,courses:list):
    check(jsessionid)
    all=deepcopy(getAllInformationFirst())
    all=all['data']["lessonTasks"]
    cids=[]
    isExit=False
    isFound=False
    for c in courses:
        isFound=False
        for one in all:
            if(one["lessonClassName"].find(c)>=0):
                cids.append(one['courseId'])
                setGlobalValue(one['courseId'],[one['electTurnLessonTaskId'],True,c,one['lessonTaskId']])
                isFound=True
                break
        if(not isFound):
            isExit=True
            print('未找到该课程: '+c)
    if(isExit):
        print('存在未找到的课程, 程序已结束运行')
    cids=tuple(cids)
    setGlobalValue('courses',cids)
    for i in cids:
        thread.start_new_thread(selectOneCourseLucky,(i,))
    time.sleep(10)
    dic={True:'选课成功',False:'选课失败'}
    getAllInformationFirst()
    for i in cids:
        print(getGlobalValue(i)[2]+': '+dic[isSelected(i)])
    os._exit(0)





